;;; gnus-fast-active.el -- active file handling speed-up for GNUS
;;; Rick Sladkey <jrs@world.std.com>

;; Here is a version of gnus-active-to-gnus-format that is 2 to 3
;; times faster than the one provided with GNUS 4.1.  For example, on
;; an active file that contained over 5000 newsgroups, this version
;; took 13 seconds while the standard version took 35 seconds.  Don't
;; forget to byte-compile it or it won't help as much.

;; One way to use it is like this:
;;   (add-hook 'gnus-startup-hook '(lambda () (require 'gnus-fast-active)))

(require 'gnus)
(provide 'gnus-fast-active)

(defun gnus-active-to-gnus-format ()
  "Convert active file format to internal format.
Lines matching gnus-ignored-newsgroups are ignored."
  ;; Delete unnecessary lines.
  (goto-char (point-min))
  ;;(delete-matching-lines "^to\\..*$")
  (delete-matching-lines gnus-ignored-newsgroups)
  ;; Save OLD active info.
  (setq gnus-octive-hashtb gnus-active-hashtb)
  ;; Make large enough hash table.
  (setq gnus-active-hashtb
	(gnus-make-hashtable (count-lines (point-min) (point-max))))
  ;; Store active file in hashtable.
  (goto-char (point-min))
  (let ((current-buffer (current-buffer))
	group min max type)
    (condition-case nil
	(while t
	  (setq group (symbol-name (read current-buffer))
		min (read current-buffer)
		max (read current-buffer)
		type (read current-buffer))
	  (and (memq type '(y m n))
	       (gnus-sethash group
			     (list group (eq type 'y) (cons max min))
			     gnus-active-hashtb)))
      (end-of-file nil))))
